#!/bin/sh
#\
exec tclsh "$0" ${1+"$@"} 

#*****************************************************************************
#
# A fast hack to get a serial output terminal at OpenBSD
# 
#-----------------------------------------------------------------------------
#
# Author: Falk Richter, 2017, falk.richter at yandex.com
#
#-----------------------------------------------------------------------------
#
# This file is licensed under the GPLv3.
#
#***************************************************************************** 

package require msgcat

############################################################################## 
# generate a new namespace
##############################################################################
namespace eval ::STerm {
  variable LOCAL

  namespace export Main 
    
  # the standard settings for the rs232 communication
  #set LOCAL(ComPort) "/dev/ttyU0"
  set LOCAL(ComPort) "/dev/ttyUSB0"
  set LOCAL(Baudrate) 115200
  set LOCAL(Parity) n
  set LOCAL(Databits) 8
  set LOCAL(Stopbits) 1
  # empty com id to signalize: com port closed 
  set LOCAL(ComId) ""

  set LOCAL(Msg) ""
}


##############################################################################
# the main procedure
##############################################################################
proc ::STerm::Main {} {
  variable LOCAL

  # open the com port
  if { [::STerm::OpenComPort] } {
	# an error occured
	puts "An error occured:"
	puts $LOCAL(ErrorMsg)
	exit 1
  }

  while {1} {
	vwait ::STerm::LOCAL(Msg)

	puts $LOCAL(Msg) 
  }
}

##############################################################################
# open the rs232 port and register the read callback
##############################################################################
proc ::STerm::OpenComPort {} {
  variable LOCAL;
  global tcl_platform

  # test if the com port already open
  if {$LOCAL(ComId) ne ""} {
	return 0
  }

  # open the serial port depending on the OS
  switch -- $tcl_platform(os) {
	"OpenBSD" -
	"NetBSD" {
	  # the hints for NetBSD are stolen from: http://wiki.tcl.tk/447
	  # open the com port for r/w access and non blocking
	  if {[catch {open $LOCAL(ComPort) {RDWR NONBLOCK}} LOCAL(ComId)]} { 
		# closing isn't necessary normally (we didn't get it open) but
		# it make senses if anybody disconnect the gauge from the rs232 port
		# during working phase and try to reconnect...
		::STerm::CloseComPort	

		# generate an error message
		set LOCAL(ErrorMsg) [msgcat::mc "An error occurred during opening the serial interface: %s" $LOCAL(ComId)] 
		return 1
      }
	  # we haven't a CARRIER DECTECT line
	  if {[catch {exec stty -f $LOCAL(ComPort) clocal} LOCAL(ReturnVal)]} {
		# generate an error message
		set LOCAL(ErrorMsg) [msgcat::mc "An error occurred during switching of the CARRIER DETECT line: %s" $LOCAL(ReturnVal)] 
		return 1
      }
    }
	"Windows NT" {
	  # open the com port for r/w access
	  if {[catch {open \\\\.\\$LOCAL(ComPort) r+} LOCAL(ComId)]} { 
		# generate an error message
		set LOCAL(ErrorMsg) [msgcat::mc "An error occurred during opening the serial interface: %s" $LOCAL(ComId)] 
		return 1
      }
    }
	"Linux" {
	  # open the com port for r/w access
	  if {[catch {open $LOCAL(ComPort) r+} LOCAL(ComId)]} { 
		# generate an error message
		set LOCAL(ErrorMsg) [msgcat::mc "An error occurred during opening the serial interface: %s" $LOCAL(ComId)] 
		return 1
      }
    }
	default {
	  # generate an error message
	  set LOCAL(ErrorMsg) [msgcat::mc "Sorry, your operation system (%s) isn't supported yet." $tcl_platform(os)] 
	  return 1
    } 
  }

  # configure the com port
  if {[catch {fconfigure $LOCAL(ComId) -blocking 0 -translation binary \
	  -mode $LOCAL(Baudrate),$LOCAL(Parity),$LOCAL(Databits),$LOCAL(Stopbits)} LOCAL(ReturnVal)]} {
		# generate an error message	
		set LOCAL(ErrorMsg) [msgcat::mc "An error occurred during configuration the serial interface: %s" $LOCAL(ReturnVal)] 
		# configuration fails we try to close the comport for a second try
		::STerm::CloseComPort
		return 1
  }

  # set the read data event
  fileevent $LOCAL(ComId) readable [list ::STerm::ReadCallback]

  return 0
}


##############################################################################
# close the rs232 port
##############################################################################
proc ::STerm::CloseComPort {} {
  variable LOCAL;

  # if the com id is empty we can break immediately
  if { $LOCAL(ComId) eq "" } {return}
  
  catch {close $LOCAL(ComId)}

  # empty com id to signalize: com port closed
  set LOCAL(ComId) "" 
}


##############################################################################
# the callback for the rs232 read function
##############################################################################
proc ::STerm::ReadCallback {} {
  variable LOCAL

  # read from the channel
  if {[catch {read $::STerm::LOCAL(ComId)} Msg]} {
    # a read error occurred
    #puts "Read error: $Msg"
    return
  }

  set LOCAL(Msg) $Msg
} 

# start Main
::STerm::Main
