/***********************************************************************
 * @file Eeprom.c
 * @brief Module reads and stores the default value of input and
 *        volume
 * @version 0.0.1
 ***********************************************************************
 * @section License
 * <b>(C) Copyright 2017 Falk Richter, falk.richter at yandex.com</b>
 ***********************************************************************
 *
 * This file is licensed under the GPLv3.
 *
 **********************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include "HAL_Settings.h"
#include "Eeprom.h"

#include "Uart1.h"

// defines for the spi status register
// spi interrupt flag
#define SPIF 0x80
// spi write collision flag
#define WCOL 0x40
// defines for the spi control register
// spi enable
#define SPEN 0x40
// spi master mode
#define MSTR 0x10
// slave select pin is ignored
#define SSIG 0x80
// clock phase (drive at leading edge, sample at falling edge)
#define CPHA 0x04
// interrupt enable register 2
// spi interrupt enable
#define EN_SPI_INT 0x02


// the EEPROM is an Microchip 25LC010A (128bit x 8) with SPI interface


// declaration of module global functions 
static void vWriteByte (uint8_t u8ByteToSend);
static uint8_t u8ReadByte (uint8_t u8ByteToSend);


#define MAX_CNT_OF_BYTES 5u
static volatile uint8_t mg_au8BytesToSend[MAX_CNT_OF_BYTES];
static volatile uint8_t mg_au8BytesReceived[MAX_CNT_OF_BYTES];
static volatile uint8_t mg_u8BytesToTransfer;
static volatile bool mg_bTransferFinished;

//----------------------------------------------------------------------
// initialize the eeprom module (startup the spi interface)
//----------------------------------------------------------------------
void EEPROM_vInit(void) {

  // first we set the write protection active
  EEPROM_WP = 0;
  
  SPCTL |=  (1u << 7u); // disable slave select
  SPCTL |=  (1u << 6u); // enable SPI
  SPCTL &= ~(1u << 5u); // MSB first
  SPCTL |=  (1u << 4u); // SPI module is master
  SPCTL &= ~(1u << 3u); // clock is low during idle and active high
  SPCTL &= ~(1u << 2u); // sampling at the first edge
  SPCTL  =  (SPCTL & ~0x03); // clear the lowest two bits for speed = cpu_clk / 4
  // clear the port switching for the standard position P1.2 to P1.5
  AUXR1 = (AUXR1 & ~(3u << 2u));

  // clear the spi status register
  SPSTAT = SPIF | WCOL;  
}


//----------------------------------------------------------------------
// read the parameter from the EEPROM
// parameters:
// pu8Input    pointer to the input line selector
// pu8Volume   pointer to the sound level
//
// return 0 if the parameter seems to be valid
//----------------------------------------------------------------------
uint8_t EEPROM_u8Read(uint8_t * pu8Input, uint8_t * pu8Volume) { 

  uint8_t u8CheckSum;
  uint8_t au8BytesReceived[3u];

  // we start with the chip select signal
  EEPROM_CS = 0;

  // read command is 3
  vWriteByte(0x03);
  
  // the address is zero
  vWriteByte(0x00);
  
  // the next three bytes are to read -> for this we send out
  // a high signal
  au8BytesReceived[0] = u8ReadByte(0xFF); 
  au8BytesReceived[1] = u8ReadByte(0xFF);
  au8BytesReceived[2] = u8ReadByte(0xFF);

  // reading is finished we disable the chip select signal
  EEPROM_CS = 1u; 

  // the first four bits of the first read byte must have 
  // the pattern 1010 (see write function)
  if ( (au8BytesReceived[0] & 0xF0) != 0xA0 ) {
	// the pattern is wrong: data is corrupt or was never written
	return(1u);
  }

  // we found the pattern so we can go further

  // we test the check sum
  u8CheckSum = (uint8_t) ( au8BytesReceived[0] + au8BytesReceived[1] ); 

  if (au8BytesReceived[2] != u8CheckSum) {
	// the checksum is wrong
	return(1u); 
  }

  // the checksum fits so we can go further

  // we copy the input byte (and remove the pattern)
  *pu8Input = (au8BytesReceived[0] & 0x0F);

  // we copy the volume information
  *pu8Volume = au8BytesReceived[1];

  return(0);
}


//----------------------------------------------------------------------
// save the parameters into the EEPROM 
//----------------------------------------------------------------------
void EEPROM_vWrite(uint8_t u8Input, uint8_t u8Volume) {

  uint8_t au8BytesToSend[3u]; 

  // first we deactivate the write protection
  EEPROM_WP = 1u;
  // and activate the chip select signal
  EEPROM_CS = 0;

  // and send out the write enable command 
  vWriteByte(0x06);

  // we take the chip select back
  EEPROM_CS = 1u;

  // to have a break between the write enable and the real
  // write operation we prepare the data to write here 

  // the first data  byte is the input, in the first four bits
  // we place the pattern 1010
  au8BytesToSend[0] = 0xA0;

  // the input information is placed in the lower four bits  
  au8BytesToSend[0] |= (u8Input & 0x0F); 

  // the third byte is the volume 
  au8BytesToSend[1] = u8Volume;

  // the last byte is the checksum
  au8BytesToSend[2] = (uint8_t) ( au8BytesToSend[0] + au8BytesToSend[1] ); 

  // now we start the transfer
  EEPROM_CS = 0;

  // write command is 2
  vWriteByte(0x02);

  // the address is zero
  vWriteByte(0x00);

  // write out the three data bytes
  vWriteByte(au8BytesToSend[0]);
  vWriteByte(au8BytesToSend[1]);
  vWriteByte(au8BytesToSend[2]);

  // take the chip select signal away
  EEPROM_CS = 1u;

  // restore the write protection again
  EEPROM_WP = 0; 
}


//----------------------------------------------------------------------
// write out the byte u8ByteToSend
//---------------------------------------------------------------------- 
void vWriteByte (uint8_t u8ByteToSend) {

  // copy byte into the transfer register
  SPDAT = u8ByteToSend;

  // wait until transfer is finished
  while ( (SPSTAT & SPIF) == 0 ) {;}

  // reset the "interrupt" flags
  SPSTAT = SPIF | WCOL;
}


//----------------------------------------------------------------------
// write out the byte u8ByteToSend and return the read spi byte
//---------------------------------------------------------------------- 
uint8_t u8ReadByte (uint8_t u8ByteToSend) {

  // copy byte into the transfer register
  SPDAT = u8ByteToSend;

  // wait until transfer is finished
  while ( (SPSTAT & SPIF) == 0 ) {;}

  // reset the "interrupt" flags
  SPSTAT = SPIF | WCOL;

  // return the read byte
  return(SPDAT);
}

