/***********************************************************************
 * @file Eeprom.c
 * @brief Module reads and stores the default value of input and
 *        volume
 * @version 0.0.1
 ***********************************************************************
 * @section License
 * <b>(C) Copyright 2017 Falk Richter, falk.richter at yandex.com</b>
 ***********************************************************************
 *
 * This file is licensed under the GPLv3.
 *
 **********************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include "HAL_Settings.h"
#include "Eeprom.h"

#include "Uart1.h"

// defines for the spi status register
// spi interrupt flag
#define SPIF 0x80
// spi write collision flag
#define WCOL 0x40
// defines for the spi control register
// spi enable
#define SPEN 0x40
// spi master mode
#define MSTR 0x10
// slave select pin is ignored
#define SSIG 0x80
// clock phase (drive at leading edge, sample at falling edge)
#define CPHA 0x04
// interrupt enable register 2
// spi interrupt enable
#define EN_SPI_INT 0x02


// the EEPROM is an Microchip 25LC010A (128bit x 8) with SPI interface


// declaration of module global functions
static void vDoTransfer(uint8_t u8FirstByte);

#define MAX_CNT_OF_BYTES 5u
static volatile uint8_t mg_au8BytesToSend[MAX_CNT_OF_BYTES];
static volatile uint8_t mg_au8BytesReceived[MAX_CNT_OF_BYTES];
static volatile uint8_t mg_u8BytesToTransfer;
static volatile bool mg_bTransferFinished;

//----------------------------------------------------------------------
// initialize the eeprom module (startup the spi interface)
//----------------------------------------------------------------------
void EEPROM_vInit(void) {

  // first we set the write protection active
  EEPROM_WP = 0;
  
  // initialize the spi data register
  SPDAT = 0; 
  
  // set the control register
  // the rest is zero means:
  // - MSB first
  // - CLK is low during idle 
  SPCTL = (SPEN | MSTR | SSIG);
  //   | CPHA 

  // clear the spi status register
  SPSTAT = SPIF | WCOL;  
  
  // enable the spi interrupt
  IE2 |= EN_SPI_INT;
}


//----------------------------------------------------------------------
// the spi interrupt
//----------------------------------------------------------------------
void EEPROM_vSpiInterrupt(void) __interrupt (9) {
  static uint8_t u8TransferIdx = 0; 

  UART1_vPrint("Eeprom\n");	
  
  
  // clear the interrupt flags
  SPSTAT = SPIF | WCOL;

  // we do it up to the second last byte
  if ( u8TransferIdx < (mg_u8BytesToTransfer - 1u) ) {
	// save the received byte
	mg_au8BytesReceived[u8TransferIdx] = SPDAT; 
	
	// now we increase the index for the next byte
	u8TransferIdx++;
	
	// start sending out the next byte
	SPDAT = mg_au8BytesToSend[u8TransferIdx];
  }
  else {
	// we have sent out all bytes...
	//
	// save the last received byte
	mg_au8BytesReceived[u8TransferIdx] = SPDAT; 
	
	// reset the index counter for the next read/write operation	
	u8TransferIdx = 0;
	
	// signalize "we are finished"
	mg_bTransferFinished = true; 
  } 
}


//----------------------------------------------------------------------
// read the parameter from the EEPROM
// parameters:
// pu8Input    pointer to the input line selector
// pu8Volume   pointer to the sound level
//
// return 0 if the parameter seems to be valid
//----------------------------------------------------------------------
uint8_t EEPROM_u8Read(uint8_t * pu8Input, uint8_t * pu8Volume) { 

  uint8_t u8CheckSum;

  // prepare the transmit array
  // read command is 3
  mg_au8BytesToSend[0] = 0x03;
  // the address is zero
  mg_au8BytesToSend[1] = 0x00;
  // the next three bytes are to read -> for this we send out zeros
  mg_au8BytesToSend[2] = 0x00;
  mg_au8BytesToSend[3] = 0x00;
  mg_au8BytesToSend[4] = 0x00;
  // we have to transfer five bytes but only four in the 
  // interrupt function
  mg_u8BytesToTransfer = 5u;
  
  // we start the transfer
  vDoTransfer(mg_au8BytesToSend[0]);

  // the first four bits of the first read byte must have 
  // the pattern 1010 (see write function)
  if ( (mg_au8BytesReceived[2] & 0xF0) != 0xA0 ) {
	// the pattern is wrong: data is corrupt or was never written
	return(1u);
  }

  // we found the pattern so we can go further

  // we test the check sum
  u8CheckSum = (uint8_t) ( mg_au8BytesReceived[2] + mg_au8BytesReceived[3] ); 
  
  if (mg_au8BytesReceived[4] != u8CheckSum) {
	// the checksum is wrong
	return(1u); 
  }

  // the checksum fits so we can go further

  // we copy the input byte (and remove the pattern)
  *pu8Input = (mg_au8BytesReceived[2] & 0x0F);

  // we copy the volume information
  *pu8Volume = mg_au8BytesReceived[3];

  return(0);
}


//----------------------------------------------------------------------
// save the parameters into the EEPROM 
//----------------------------------------------------------------------
void EEPROM_vWrite(uint8_t u8Input, uint8_t u8Volume) {

  // first we deactivate the write protection
  EEPROM_WP = 1u;

  // and send out the write enable command 

  // we send out only one byte
  mg_u8BytesToTransfer = 1u;

  // we send out the single command: write enable
  vDoTransfer(0x06); 

  // to have a break between the write enable and the real
  // write operation we prepare the data to write here

  // write command is 2
  mg_au8BytesToSend[0] = 0x02;
  
  // the address is zero
  mg_au8BytesToSend[1] = 0x00;
  
  // the next byte is the input, in the first four bits
  // we place the pattern 1010
  mg_au8BytesToSend[2] = 0xA0;
  
  // the input information is placed in the lower four bits  
  mg_au8BytesToSend[2] |= (u8Input & 0x0F); 
  
  // the third byte is the volume 
  mg_au8BytesToSend[3] = u8Volume;
  
  // the last byte is the checksum
  mg_au8BytesToSend[4] = (uint8_t) ( mg_au8BytesToSend[2] + mg_au8BytesToSend[3] ); 

  // we five bytes to transfer 
  mg_u8BytesToTransfer = 5u;

  // we start the transfer
  vDoTransfer(mg_au8BytesToSend[0]);

  // restore the write protection again
  EEPROM_WP = 0; 
} 


//----------------------------------------------------------------------
// start the spi transfer and wait for finishing
//----------------------------------------------------------------------
void vDoTransfer(uint8_t u8FirstByte) {

  // we start the transfer with a low level on the chip select pin
  EEPROM_CS = 0;

  // reset the finished flag
  mg_bTransferFinished = false; 
  
  // we send out the first byte
  SPDAT = u8FirstByte;
    
  // reduce the count of bytes to transfer by one
  if ( mg_u8BytesToTransfer > 0 ) {
	mg_u8BytesToTransfer--;
  } 
 
  // and wait until finishing
  while (mg_bTransferFinished != true) {;}
  
  // we reset the chip select pin to high
  EEPROM_CS = 1u; 
}

