/**********************************************************************
 * @file Timer0.c
 * @brief Timer 0 is used for 100us clock and as delay timer
 * @version 0.0.1
 ***********************************************************************
 * @section License
 * <b>(C) Copyright 2017 Falk Richter, falk.richter at yandex.com</b>
 ***********************************************************************
 *
 * This file is licensed under the GPLv3.
 *
 **********************************************************************/

#include <stdint.h>
#include <stdbool.h>

#include "HAL_Settings.h"
#include "Timer0.h"

// global 100us timer tick flag
bool TIMER0_b100usTimeTick;


// delay timer counter variable
static uint8_t mg_u8Counter;

// sub function of the delay timer
static uint8_t u8GetDiff (uint8_t u8StartValue);

//----------------------------------------------------------------------
// initialize timer 0 to generate 100us ticks
//----------------------------------------------------------------------
void TIMER0_vInit(void) { 
  // systick timer runs with 1/10000=100us
  const uint16_t u16SysTick = 10000u;
  
  // first calculate the timer reload value
  // value = osc / systick + 0.5 (0.5 to round to the next full integer after casting)
  // here is the formula written to never leave the integer range
  const uint32_t u32TimerClocks = ( UC_u32OscFreq + u16SysTick / 2u ) / u16SysTick; 
  const uint16_t u16TimerReload =	(uint16_t) (((uint32_t) 65536u) - u32TimerClocks); 

  AUXR = 0x80;	//Timer0 set as 1T, 16 bits timer auto-reload, 
  TH0 = (uint8_t) (u16TimerReload / 256u);
  TL0 = (uint8_t) (u16TimerReload % 256u);
  ET0 = 1;	// Timer0 interrupt enable
  TR0 = 1;	// run Tiner0

  // reset the timer tick flag
  TIMER0_b100usTimeTick = false;
}


//----------------------------------------------------------------- 
// A blocking delay timer
//
// u8Time
// Delay time as factor of 100us (1 = 100us delay) 
// 
// ----------------------------------------------------------------
void TIMER0_vDelay (uint8_t u8Time) {
  uint8_t u8StartVal;

  // the time value must be between 1 and 255
  if (u8Time == 0) {
	// no time delay
	return;
  }
  
  // get the current value of the tick timer as start value
  u8StartVal = mg_u8Counter;

  while ( u8GetDiff( u8StartVal ) < u8Time);
}


//----------------------------------------------------------------- 
// Function delivers the difference between the current value of 
// the time tick counter and the overtaken start value
//
// u8StartVal: start value
// 
// returns: time tick difference value
// ---------------------------------------------------------------- 
uint8_t u8GetDiff (uint8_t u8StartVal) {
  uint8_t u8CurrentVal;
  uint8_t u8ReturnVal;

  // get the current value of the tick timer
  // we make a copy for an atomic access
  u8CurrentVal = mg_u8Counter;

  if ( u8CurrentVal >= u8StartVal ) { 
    // no overflow of the tick timer -> we can return the simple difference
    u8ReturnVal = u8CurrentVal - u8StartVal;
  } else {
    // the current value is smaller than the start value
    // -> the timer must be overflown
    // give back 255 - StartValue + CurrentValue + 1 (zero is also one step)
    u8ReturnVal = (uint8_t) (256u - u8StartVal + u8CurrentVal);
  }
 
  return( u8ReturnVal );
}


//----------------------------------------------------------------------
// Timer0 interrupt
// ---------------------------------------------------------------------
void TIMER0_vInterrupt(void) __interrupt (1)
{ 
  // 100us are over	
  TIMER0_b100usTimeTick = true; 

  // the counter for the delay timer
  if ( mg_u8Counter < 255u ) {
    mg_u8Counter++;
  } else {
    mg_u8Counter = 0;
  }
}

