/***********************************************************************
 * Display.c
 * Handle the content of the Lcd1602a display
 * 
 ***********************************************************************
 * License
 * (C) Copyright 2018 Falk Richter, falk.richter at yandex.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/


#include <stdint.h>
#include <stdbool.h>
#include "Lcd1602a.h"
#include "Timer0.h"
#include "Uart1.h"
#include "Display.h" 

//---------------------------------------------------------------------- 
// initialize the display and show the startup screen on it
//---------------------------------------------------------------------- 
void DIS_vShowStartupScreen(void) {

  // initialize the display
#ifdef DEBUG 
  if ( LCD1602A_u8Init() ) { 
	UART1_vPrint("Lcd initialization error.\n");	
  }
#else
  (void) LCD1602A_u8Init();
#endif

  // write the start screen
  (void) LCD1602A_u8WriteString(0,0," Falk's  PreAmp "); 
  (void) LCD1602A_u8WriteString(1,0,"Version 0.1 2017");

  // we show the start screen for 1s
  TIMER0_vDelay(2u, e1s);  

  // write the standard screen
  (void) LCD1602A_u8WriteString(0,0,"Input:          "); 
  (void) LCD1602A_u8WriteString(1,0,"Volume:         "); 
} 


//---------------------------------------------------------------------- 
// show the shut down screen on the display and switch off it
//---------------------------------------------------------------------- 
void DIS_vShowShutDownScreen(void) {
// write the shut down screen
  (void) LCD1602A_u8WriteString(0,0,"Have a nice day!"); 
  (void) LCD1602A_u8WriteString(1,0,"     Goodby     ");
  
  // we show the start screen for 1s
  TIMER0_vDelay(2u, e1s);

  // switch off the display
  LCD1602A_vSwitchOff(); 
}


//---------------------------------------------------------------------- 
// Switch off the display
//---------------------------------------------------------------------- 
void DIS_vSwitchOff(void) {
  // we only pass the request to the sub-module
  LCD1602A_vSwitchOff(); 
} 


//----------------------------------------------------------------------
// show the new sound level on the display
// ---------------------------------------------------------------------
void DIS_vUpdateDisplayVolume (uint8_t u8NewSoundLevel) {
  
  bool bHigestDigitPresent = false;
  uint8_t u8DigitAsAscii; 
   
  // on both ends (127 and 0) we show MIN and MAX instead numbers
  if ( u8NewSoundLevel == 127u ) {
	(void) LCD1602A_u8WriteString(1,13u,"MAX");
	return;
  }
  
  if ( u8NewSoundLevel == 0 ) {
	(void) LCD1602A_u8WriteString(1,13u,"MIN");
	return;
  } 
  
  // we parse the highest digit
  if ( u8NewSoundLevel > 99u ) { 
	
	// it is >= 100 we set the one
	LCD1602A_u8WriteChar(1u ,13u ,'1');

	// first digit is done we prepare the sound level
	// for the next digit
	u8NewSoundLevel -= 100u;
	// we set a flag that we have a highest digit
	bHigestDigitPresent = true;
  } else {
	// the value is smaller than 100 -> we place a space
	LCD1602A_u8WriteChar(1u ,13u , ' ');
  } 
  
  // we parse the middle digit (the flag is necessary because
  // we modified u8NewSoundLevel e.g. SoundLevel is 105 -> only 
  // 5 is left here)
  if ( bHigestDigitPresent ||  (u8NewSoundLevel > 9u) ) {
	// the middle digit is present, we separate it 
	// and convert it to ASCII
	u8DigitAsAscii = (u8NewSoundLevel / 10u) + 0x30;	
	// we write the middle digit
	LCD1602A_u8WriteChar(1u, 14u, u8DigitAsAscii); 
  } else {
	// the value is smaller than 10 and smaller than 100 -> 
	// we place a space
	LCD1602A_u8WriteChar(1u, 14u, ' ');
  }

  // the lowest digit we show always
  u8DigitAsAscii = (u8NewSoundLevel % 10u) + 0x30;	
  LCD1602A_u8WriteChar(1u, 15u, u8DigitAsAscii); 
}


//----------------------------------------------------------------------
// show the new line input on the display
// ---------------------------------------------------------------------
void DIS_vUpdateDisplayInput (uint8_t u8NewInput) {
  
  switch (u8NewInput) {
	case 1u:
	  {
		LCD1602A_u8WriteString(0, 11u, "   CD");
		break;
	  }
	case 2u:
	  {
		LCD1602A_u8WriteString(0, 11u, "PHONO");
		break;
	  }
	case 3u:
	  {
		LCD1602A_u8WriteString(0, 11u, "  DAC");
		break;
	  } 
	case 4u:
	  {
		LCD1602A_u8WriteString(0, 11u, "  AUX");
		break;
	  } 
  }
}

