/***********************************************************************
 * Eeprom.c
 * Module reads and stores the default value of input and
 * volume
 * 
 ***********************************************************************
 * License
 * (C) Copyright 2018 Falk Richter, falk.richter at yandex.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include "HAL_Settings.h"
#include "Eeprom.h"
#include "Timer0.h"

// the EEPROM is an old Atmel (Microchip) 93C46 with SPI
// (other manufacturers: ST, On-Semi, Fremont, Seiko/Ablic...)
// we make the SPI-interface in software (and don't 
// use the hardware module) because the protocol isn't byte 
// oriented (9 bits commands). But the implementation  doesn't 
// generate performance problems because
// - we read only once after switching on
// - we write only once after switching off (and than only
//   if anything was changed)

// declaration of module global functions
static void vSendBits(uint32_t u32, uint8_t u8BitCnt);
static uint16_t u16ReadBits(void); 

//----------------------------------------------------------------------
// initialize the eeprom module
//----------------------------------------------------------------------
void EEPROM_vInit(void) {

  // deactivate chip select
  EEPROM_CS = 0;
  // preset Master out and clk to low
  EEPROM_MO = 0;
  EEPROM_CLK = 0;
}


//----------------------------------------------------------------------
// read the parameter from the EEPROM
// parameters:
// pu8Input    pointer to the input line selector
// pu8Volume   pointer to the sound level
//
// return 0 if the parameter seems to be valid
//----------------------------------------------------------------------
uint8_t EEPROM_u8Read(uint8_t * pu8Input, uint8_t * pu8Volume) {

  uint16_t u16Data;
  uint8_t u8Input; 
 
  // activate chip select
  EEPROM_CS = 1u;

  // send the read command (110) and address zero (0 0000 0) ->
  // 1100 0000 0  
  vSendBits(0xC0000000, 9u);

  // read the data bytes
  u16Data = u16ReadBits();

  // reset the chip select
  EEPROM_CS = 0;

  // make the CS valid
  TIMER0_vDelay(1u, e100us); 

  // we separate the first bit
  u8Input = (uint8_t) (u16Data >> 8u);

  // the first four bits must have the pattern 1010 (see write function)
  if ( (u8Input & 0xF0) != 0xA0 ) {
	// the pattern is wrong: data is corrupt or was never written
	return(1u);
  }

  // we found the pattern so we can go further

  // we copy the input byte (and remove the pattern)
  *pu8Input = (u8Input & 0x0F);

  // we copy the volume information
  *pu8Volume = (uint8_t) (u16Data & 0x00FF);

  return(0);
}


//----------------------------------------------------------------------
// save the parameters into the EEPROM 
//----------------------------------------------------------------------
void EEPROM_vWrite(uint8_t u8Input, uint8_t u8Volume) {

  // holds the bits to write
  uint32_t u32Data;

  // first we assemble the data bytes
  uint16_t u16Data;

  // in the first byte we save the input selector which has only a value range
  // from 1...4 so we have space for a "1010" pattern in the first four (MSB) bits
  // which we test later in the read function
  u16Data = 0xA000;
  // the input data can be 1...4
  u16Data |= ((uint16_t) (u8Input & 0x0F)) << 8u;
  // the second byte saves the volume information
  u16Data |= u8Volume;

  // start the transfer
  // activate the chip select
  EEPROM_CS = 1u;

  // send the write enable command
  // the write enable command has the sequence: 1001 1000 0
  vSendBits(0x98000000, 9u);

  // reset the chip select
  EEPROM_CS = 0;

  // chip select hold time
  TIMER0_vDelay(1u, e100us); 

  // we assemble the data bits to send 
  u32Data = (uint32_t) u16Data;

  // after the start bit are 2 bits for the command, 6 bits address plus 16 bit data
  // the last 7 bits are unused
  // move the content of u16Data (see above)
  u32Data <<= 7u;
  // and added the command 
  // (write command (101) and address 0 (0 0000 0) -> 1010 0000 0)
  u32Data |= 0xA0000000;

  // activate the chip select again
  EEPROM_CS = 1u;

  // send the 
  vSendBits(u32Data, 25u);

  // reset CS
  EEPROM_CS = 0;

  // saving data into the EEPROM could take up to 10ms
  // we chose two here because the first period with this counter
  // can be 0...10ms
  TIMER0_vDelay(2u, e10ms);

  // activate the chip select again
  EEPROM_CS = 1u;

  // send the write disable command

  // the write disable command has the sequence: 1000 0000 0
  vSendBits(0x80000000, 9u);

  // reset the chip select
  EEPROM_CS = 0;

  // chip select hold time
  TIMER0_vDelay(1u, e100us);
}


//----------------------------------------------------------------------
// sub-function to write the bits
//----------------------------------------------------------------------
void vSendBits(uint32_t u32Data, uint8_t u8BitCnt) {

  // loop variable
  uint8_t u8i;
  // variable to mask the bits in u32Data
  uint32_t u32Mask = 0x80000000;

  // we can't send more than 25 bits
  if (u8BitCnt > 25u) {
	return;
  } 

  // we start with a delay (after CS goes to high)
  TIMER0_vDelay(1u, e100us); 

  // loop over all bits to send
  for (u8i = 0; u8i < u8BitCnt; u8i++) { 
	// set the data output bit
	if (u32Data & u32Mask) {
	  EEPROM_MO = 1u;
	} else {
	  EEPROM_MO = 0;
	}
	// data stabilization time
	TIMER0_vDelay(1u, e100us);
	// generate a low -> high clock slope
	EEPROM_CLK = 1u;
	// data hold time
	TIMER0_vDelay(1u, e100us);
	// reset the clock line
	EEPROM_CLK = 0; 
	// prepare the next bit (or better move
	// the bit mask) but not for the last bit	
	u32Mask >>= 1u; 
  } 
  // before we leave the function we make a break for clock=low
  TIMER0_vDelay(1u, e100us); 
}


//----------------------------------------------------------------------
// sub-function to read 16 bits
//----------------------------------------------------------------------
uint16_t u16ReadBits(void) {

  // loop variable
  uint8_t u8i;
  // data to read
  uint16_t u16Data = 0;

  // set the data output always to zero
  EEPROM_MO = 0;

  // loop over all bits to read
  for (u8i = 0; u8i < 16u; u8i++) { 
	// generate a low -> high clock slope
	EEPROM_CLK = 1u;
	// data stabilization time	
	TIMER0_vDelay(1u, e100us);
	// read the input pin
	if (EEPROM_MI) {
	  u16Data |= 1u;
	}
	// prepare for reading the next bit
	// with exception of the last bit
	if (u8i != 15u) {
	  u16Data <<= 1u;
	}
	// set clock back to zero	
	EEPROM_CLK = 0;
	// clock stabilization time
	TIMER0_vDelay(1u, e100us); 
  } 

  return(u16Data);
}

