/***********************************************************************
 * IR_Remote.c
 * Contains a simple NEC infrared receiver inspired by sample code
 * of STC but completely new written
 *
 ***********************************************************************
 * License
 * (C) Copyright 2018 Falk Richter, falk.richter at yandex.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

// include the header of the sdcc compiler
#include <stdint.h>
#include <stdbool.h>

// include our own header
#include "HAL_Settings.h"
#include "Timer0.h"
#include "IrNecProtocol.h"
#include "Uart1.h"
#include "Display.h"
#include "Poti.h"
#include "InputSelect.h"
#include "Eeprom.h"
#include "RotaryEnc.h" 

// the module global functions
static void vSwitchOn(void);
static void vSwitchOff(void);
static bool bCheckLvd(void);

//----------------------------------------------------------------------
// the main procedure
// ---------------------------------------------------------------------
void main(void) { 
  // local variables
  uint16_t u16IrAdr = 0;
  uint8_t u8IrCmd = 0;
  NEC_enCommands_t enIrCmdParsed = eNotValid;
  uint8_t u8IrReturnVal;
  uint8_t u8EncReturnVal;

  bool bPreAmpIsOn = false;

  // initialize the general hardware abstract layer
  HAL_vInit(); 

  // initialize timer 0 for the 100us system tick
  TIMER0_vInit();

  // enable all interrupts (system tick uses an interrupt)
  EA = 1; 

  // we deactivate the display here and wait for the switch on command
  DIS_vSwitchOff(); 

  // initialize the UART1 for debugging
#ifdef DEBUG
  UART1_vConfig();
#endif

  // initialize the spi interface for EEPROM access
  EEPROM_vInit(); 

  // initialize the rotary switch
  ROT_vInit();

#ifdef DEBUG 
  UART1_vPrint("Start infrared remote control handling:\n");	
#endif 

  while(1)
  {
    // wait for the next 100us tick
    while (TIMER0_b100usTimeTick == false) {;}
	// reset the 100us tick flag
	TIMER0_b100usTimeTick = false;

	// if the preamplifier is on - we check if we have an
	// under voltage situation
	if ( bPreAmpIsOn ) {
	  if ( bCheckLvd() ) {
		// low voltage situation -> the preamplifier was
		// switched off
		bPreAmpIsOn = false; 
	  }
	} 

	// 100us are over we call the ir read function
    u8IrReturnVal = NEC_u8IrRx( &u16IrAdr, &u8IrCmd ); 
    
	if ( u8IrReturnVal ) {
      // the ir function returns > 0 -> a full protocol or a repeated command was received
      // parse the command
      enIrCmdParsed = NEC_enIrCmdParsing(u16IrAdr, u8IrCmd);
      
	  // if the preamplifier is off - we accept only the on/off command
	  if ( bPreAmpIsOn == false ) {
		if ( enIrCmdParsed != eSwitchOnOff ) {
		  // we ignore all other commands and set it to not valid
		  enIrCmdParsed = eNotValid;
		} 
	  } 

	  // process the command
	  switch (enIrCmdParsed) {
		case eNotValid:
		  { 
#ifdef DEBUG 
			UART1_vPrint("Command not valid\n");
#endif
			// nothing to do here
			break;
		  }	
		case eSwitchOnOff:
		  {
#ifdef DEBUG 
			UART1_vPrint("Switch on/off\n");
#endif
			if (bPreAmpIsOn) {
			  // preamplifier is on - we switch it off
			  bPreAmpIsOn = false;
			  // handle all tasks to switch the preamp off
			  vSwitchOff(); 
			} else {
			  // the preamplifier is off - we switch it on
			  bPreAmpIsOn = true;
			  // execute all tasks which are needed for startup
			  vSwitchOn();
			}
			break;
		  }
		case eVolInc:
		  {
#ifdef DEBUG 
			UART1_vPrint("Vol+\n"); 
#endif
			// set the sound level relays
			POTI_vChangeSoundLevel( eIncrSoundLevel, u8IrReturnVal);
			// show the new sound level on the display
			DIS_vUpdateDisplayVolume( POTI_u8GetSoundLevel() );
			break;
		  }
		case eVolDec:
		  {
#ifdef DEBUG 
			UART1_vPrint("Vol-\n"); 
#endif
			// set the sound level relays
			POTI_vChangeSoundLevel( eDecrSoundLevel, u8IrReturnVal);
			// show the new sound level on the display
			DIS_vUpdateDisplayVolume( POTI_u8GetSoundLevel() );
			break;
		  }
		case eInput1:
		  {
#ifdef DEBUG 
			UART1_vPrint("Input 1\n"); 
#endif
			// set the new input
			INPUTSEL_vSet(1u);
			DIS_vUpdateDisplayInput( INPUTSEL_u8Get() ); 
			break;
		  }
		case eInput2:
		  {
#ifdef DEBUG
			UART1_vPrint("Input 2\n"); 
#endif
			// set the new input
			INPUTSEL_vSet(2u);
			DIS_vUpdateDisplayInput( INPUTSEL_u8Get() );
			break;
		  }
		case eInput3:
		  {
#ifdef DEBUG
			UART1_vPrint("Input 3\n"); 
#endif
			// set the new input
			INPUTSEL_vSet(3u);
			DIS_vUpdateDisplayInput( INPUTSEL_u8Get() );
			break;
		  }
		case eInput4:
		  {
#ifdef DEBUG
			UART1_vPrint("Input 4\n"); 
#endif
			// set the new input
			INPUTSEL_vSet(4u);
			DIS_vUpdateDisplayInput( INPUTSEL_u8Get() );
			break;
		  }
		case eMute:
		  {
#ifdef DEBUG
			UART1_vPrint("Mute\n"); 
#endif
			// set the sound level to zero
			//POTI_vSetSoundLevel(0);	
			// the command type is always full protocol -> 1u	
			POTI_vChangeSoundLevel( eMuteSoundLevel, 1u);	
			// show the new sound level on the display
			DIS_vUpdateDisplayVolume( POTI_u8GetSoundLevel() );
			break;
		  }
		default:
		  {
			// unknown command -> programmer error,
			// should never happen!
#ifdef DEBUG
			UART1_vPrint("Unknown command\n"); 
#endif
			break;
		  }
	  } 
	  // after parsing we reset the timer0 flag again for a new clean cycle 
	  TIMER0_b100usTimeTick = false; 
	}
	else {
	  // no infrared remote control command received we look on 
	  // rotary encoder
	  u8EncReturnVal = ROT_u8Encoder(); 

	  // if the preamplifier is off - we accept only the on/off command
	  if ( bPreAmpIsOn == false ) {
		if ( u8EncReturnVal != 4u ) {
		  // we ignore all other commands and set it to no activity
		  u8EncReturnVal = 0;
		}
	  }

	  switch (u8EncReturnVal) {
		case 0: 
		  {
			// no activity, we return immediately
			break;
		  }
		case 1u: 
		  {
			// clockwise rotated
			// increase the volume level

			// set the sound level relays
			// the command type is always full protocol -> 1u	
			POTI_vChangeSoundLevel( eIncrSoundLevel, 1u);
			// show the new sound level on the display
			DIS_vUpdateDisplayVolume( POTI_u8GetSoundLevel() );
			break;
		  }
		case 2u: 
		  {
			// counter clockwise rotated
			// decrease the volume level

			// set the sound level relays
			// the command type is always full protocol -> 1u	
			POTI_vChangeSoundLevel( eDecrSoundLevel, 1u);
			// show the new sound level on the display
			DIS_vUpdateDisplayVolume( POTI_u8GetSoundLevel() ); 
			break;
		  } 
		case 3u: 
		  {
			// the knob was pressed a short time
			// select the next input channnel
			INPUTSEL_vNextChannel();
			// update the display
			DIS_vUpdateDisplayInput( INPUTSEL_u8Get() ); 
			break;
		  }
		case 4u:
		  {
			// the knob was pressed a long time
			// switch on/off the preamplifier
			if (bPreAmpIsOn) {
			  // preamplifier is on - we switch it off
			  bPreAmpIsOn = false;
			  // handle all tasks to switch the preamp off
			  vSwitchOff(); 
			} else {
			  // the preamplifier is off - we switch it on
			  bPreAmpIsOn = true;
			  // execute all tasks which are needed for startup
			  vSwitchOn();
			}
			break;
		  } 
	  }	
	} // end of the rotary part

	// refresh the volume relays if the preamp is on
	if ( bPreAmpIsOn ) {
	  POTI_vDoWork();
	}
  }
} 


//----------------------------------------------------------------------
// handle all tasks to switch the preamp on
// --------------------------------------------------------------------- 
void vSwitchOn(void) {

  uint8_t u8DefaultSoundLevel;
  uint8_t u8DefaultInput;

  // set the on signal for the 12V (audio) power supply
  // (reversed logic)
  PWR_ON = 0; 
  
  // try to read the default values from the EEPROM
  if ( EEPROM_u8Read( &u8DefaultInput, &u8DefaultSoundLevel ) ) {
#ifdef DEBUG
	    UART1_vPrint("EEPROM read error\n"); 
#endif	
	// in error case we set the default values here
	u8DefaultSoundLevel = 30u;
	u8DefaultInput = 1u; 
  }

  // initialize the relay potentiometer
  POTI_vInit( u8DefaultSoundLevel );

  // initialize the line input selector
  INPUTSEL_vInit( u8DefaultInput ); 

  // show the startup screen 
  DIS_vShowStartupScreen();

  // show the default sound level
  DIS_vUpdateDisplayVolume( POTI_u8GetSoundLevel() );
  // show the default input
  DIS_vUpdateDisplayInput( INPUTSEL_u8Get() );

  // we give the plop protection free
  // (reversed logic) 
  PLOP_PROTECT = 0;
}


//----------------------------------------------------------------------
// handle all tasks to switch the preamp off
// --------------------------------------------------------------------- 
void vSwitchOff(void) {

  uint8_t u8SoundLevel;
  uint8_t u8Input;
  uint8_t u8ReturnVal;

  // first we activate the plop protection
  // (reversed logic) 
  PLOP_PROTECT = 1u;
  
  // try to read the values from the EEPROM
  u8ReturnVal = EEPROM_u8Read( &u8Input, &u8SoundLevel );

  // we check if there was an error during reading
  // or if we have new values
  if ( u8ReturnVal                             ||
	  (POTI_u8GetSoundLevel() != u8SoundLevel) ||
	  (INPUTSEL_u8Get() != u8Input) ) { 
	
	// we have to save the new values
	EEPROM_vWrite(INPUTSEL_u8Get(), POTI_u8GetSoundLevel());
  }

  // show the shut down screen and switch off the display
  DIS_vShowShutDownScreen();

  // set all relays into standby mode to save energy
  INPUTSEL_vSetStandBy();
  POTI_vSetStandBy();

  // take away also the on signal for the 12V (audio) power supply
  // (reversed logic) 
  PWR_ON = 1u;
}


//----------------------------------------------------------------------
// check the under voltage situation and switch off the amplifier 
// without EEPROM saving (to avoid an unfinished write process)
// --------------------------------------------------------------------- 
bool bCheckLvd(void) {

  bool bReturnValue = false;

  if ( PCON & HAL_u8LvdFlag ) {
	// low voltage detection is set -> we switch off anything
	// first we activate the plop protection
	// (reversed logic) 
	PLOP_PROTECT = 1u;
	
	// reset the low voltage flag
	PCON &= ~HAL_u8LvdFlag;
	
	// set the return value
	bReturnValue = true;
	
	// switch off the display
	DIS_vSwitchOff();
	
	// take away the on signal for the 12V (audio) power supply
	// (reversed logic) 
	PWR_ON = 1u;
  }

  return(bReturnValue);
}

