/***********************************************************************
 * IrNecProtocol.c
 * NEC (today Renesas) infrared remote control protocol
 * HAL and MAC layer
 * 
 ***********************************************************************
 * License
 * (C) Copyright 2018 Falk Richter, falk.richter at yandex.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

// include the header of the sdcc compiler
#include <stdint.h>
#include <stdbool.h>

// include our own header
#include "HAL_Settings.h"
#include "IrNecProtocol.h"


// NEC infrared remote control protocol
//
// standard protocol format (total 32 bits):
// sync-pulse, address, not_address, data, not_data
// 
// extended protocol format: 
// sync-pulse, address_low, address_high, data, not_data
// 
// repeated command:
// sync-pulse, 560us high pulse
//
// data rate: 108ms / frame 


// define the NEC protocol parameters as constants 
// bit count of the NEC protocol 
static const uint8_t u8NecProtBitCnt = 32u;
// maximum synchronization pulse length
// for a normal command: 13.5ms
// for a repeated command: 11.25ms
// -> so we select 15ms (or 150 100us loops) for time out
static const uint8_t u8NecProtSyncMax = 150u;
// a valid synchronisation pulse length
// -> we select 12.5ms (or 125 100us loops)
static const uint8_t u8NecProtSyncValid = 125u;
// a valid repeated synchronisation pulse length
// -> we select 11ms (or 110 100us loops)
static const uint8_t u8NecProtRepeatSyncValid = 110u;
// a valid low bit is 1.12ms, a valid high bit is 2.25ms long
// -> we select 3ms (or 30 100us loops)
static const uint8_t u8NecProtCmdMax = 30u;
// -> we select 1.5ms to detect a high bit
static const uint8_t u8NecProtCmdHigh = 15u; 


//----------------------------------------------------------------------
// samples the ir receiver input pin each 100us
//
// parameters:
// pu16Adr - pointer to address to return the received NEC protocol
//           address
//
// pu8Cmd  - pointer to command to return the received NEC protocol
//           command
//
// returns: 
//         - 0  if a part of a protocol was received or an error occurred
//         - 1u if a full protocol was received
//         - 2u if a repeated command was received
//
// ---------------------------------------------------------------------
uint8_t NEC_u8IrRx(uint16_t * pu16Adr, uint8_t * pu8Cmd) { 
  // counter for the samples on the input pin (100us cycle)
  static uint8_t u8SampleCounter = 0; 

  // save the last read bit
  static uint8_t u8LastBit = 0;

  // counter for the 32 bit of the NEC protocol
  static uint8_t u8IrBitCounter;
  // a valid synchronisation signal was received
  static bool bIrSyncReceived = false;
  // variable to read in 8 bits of the protocol
  static uint8_t u8IrDataShift;
  
  // address byte or low address byte at the extended NEC protocol
  static uint8_t u8IrAdrLow;
  // address-not byte or high address byte at the extended NEC protocol
  static uint8_t u8IrAdrHigh;
  // byte which contains the command
  static uint8_t u8IrCmd;
  
  // temporary variables  
  // temporary variable to save the currently read bit
  uint8_t u8CurrentBit;

  // return value
  uint8_t u8ReturnVal = 0; 

  // temporary variable for bitwise not operation
  uint8_t u8NotIrCmd;

  // we don't let u8SampleCounter overflow to have a high
  // value for the first byte
  if (u8SampleCounter < 255u) {
	// increment sample counter until 255
	u8SampleCounter++;		
  } 
  
  // read a new bit
  u8CurrentBit = PIN_IR_RX;
  
  // test for a falling edge, the infrared receiver 
  // Vishay TSOP4838 delivers a negated signal
  if (u8LastBit && !u8CurrentBit) { 
	// test for a invalid synchronisation pulse
	if (u8SampleCounter > u8NecProtSyncMax) {
	  // reset the sync flag
	  bIrSyncReceived = false; 
	}
	// test for synchronization signal received	
	else if(u8SampleCounter >= u8NecProtSyncValid) {
	  // save the flag for later usage
	  bIrSyncReceived = true;
	  // reload the full bit count
	  u8IrBitCounter = u8NecProtBitCnt;
	  // reinitialize the data bit shift register with zero
	  u8IrDataShift = 0;	
	}
    // test for repeated synchronisation pulses -> a button
	// on the remote control is kept pressed
	else if (u8SampleCounter >= u8NecProtRepeatSyncValid) {
	  // reset the sync flag because we don't expect any data
	  bIrSyncReceived = false;
	  // repeat the last received command
	  *pu16Adr = (((uint16_t) u8IrAdrHigh) << 8u) + u8IrAdrLow;
	  *pu8Cmd = u8IrCmd;
	  // a full protocol was received
	  u8ReturnVal = 2u; 
	}
	else if(bIrSyncReceived) { 
	  // has received SYNC
	  if(u8SampleCounter > u8NecProtCmdMax) {
		// the command time is too large
		bIrSyncReceived = false; 
	  }
	  else {
		//shift to set the next bit
		u8IrDataShift >>= 1;
		
		if(u8SampleCounter >= u8NecProtCmdHigh) {
	  	  //the bit is high - we set it
		  //in the NEC protocol LSB is coming first
		  //so we convert here also endian to MSB first
		  u8IrDataShift |= 0x80; 
		}
		
		if(--u8IrBitCounter == 0) { 
		  // all bits received -> clear sync flag
		  bIrSyncReceived = false;
		  // check the correctness of the received data bytes
		  // ~OfNegation == NotNegation
		  // we need here a temporary variable for the sdcc compiler
		  // because in "if (~u8IrDataShift == u8IrCmd)" the negation doesn't work
		  u8NotIrCmd = ~u8IrDataShift; 

		  if(u8NotIrCmd == u8IrCmd) {
			// Byte 3 is equal to not Byte 4
			*pu16Adr = (((uint16_t) u8IrAdrHigh) << 8u) + u8IrAdrLow;
			*pu8Cmd = u8IrCmd;
			// a full protocol was received
			u8ReturnVal = 1u;
		  }
		}
		else if((u8IrBitCounter & 0x07) == 0) {
		  //one byte receive
		  //save the byte in form of a shift register 
		  u8IrAdrLow = u8IrAdrHigh;
		  u8IrAdrHigh = u8IrCmd;
		  u8IrCmd  = u8IrDataShift;
		}
	  }
	}
  
	// reset the sample counter after an edge
	u8SampleCounter = 0; 
  }

  // save the last sampled bit
  u8LastBit = u8CurrentBit;
  
  return(u8ReturnVal);
}


//----------------------------------------------------------------------
// parse the received protocol frame
//
// parameters:
// u16Adr - address received over the NEC protocol
//
// u8Cmd  - command received over the NEC protocol
//
// returns the transmitted command or "command invalid" if the command
//         is unknown or address doesn't fit 
// --------------------------------------------------------------------- 
NEC_enCommands_t NEC_enIrCmdParsing(const uint16_t u16Adr, const uint8_t u8Cmd) {

  NEC_enCommands_t enReturnVal;

  // check the correctness of the address
  if (u16Adr != 0xFF00) {
	// address is not from our remote transmitter
	return(eNotValid); 
  }

  // parse the command
  switch (u8Cmd) {
	case 0x45:
	  {
		enReturnVal = eSwitchOnOff;
		break; 
	  }
	case 0x40:
	  {
		enReturnVal = eVolInc;
		break; 
	  }
	case 0x19:
	  {
		enReturnVal = eVolDec;
		break; 
	  } 
	case 0x0C:
	  { 
		enReturnVal = eInput1;
		break;
	  }
	case 0x18:
	  { 
		enReturnVal = eInput2;
		break;
	  } 
	case 0x5E:
	  { 
		enReturnVal = eInput3;
		break;
	  }
	case 0x08:
	  { 
		enReturnVal = eInput4;
		break;
	  }
	case 0x47:
	  {
		enReturnVal = eMute;
		break; 
	  }	
	default:
	  {
		// unknown command or a command
		// which we don't support
		enReturnVal = eNotValid; 
		break;
	  }
  }

  return(enReturnVal);
}

