/***********************************************************************
 * Lcd1602a.c
 * Driver for the 16 x 2 character display LCD1602A
 * 
 ***********************************************************************
 * License
 * (C) Copyright 2018 Falk Richter, falk.richter at yandex.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

// include the header of the sdcc compiler
#include <stdint.h>
#include <stdbool.h>

// include our own header
#include "HAL_Settings.h"
#include "Timer0.h"
#include "Lcd1602a.h" 

// local functions declaration
static void vDelay500ns(void);
static uint8_t u8SendCmd(uint8_t u8Cmd, bool bWithBusyCheck);
static uint8_t u8SendDisplayData(uint8_t u8Data);

// local variables
static const uint8_t mg_u8MaxRows = 2u;
static const uint8_t mg_u8MaxColumns = 16u;

//----------------------------------------------------------------------
// setup function for the LCD1602A display
//
// returns 0 in case of success
// --------------------------------------------------------------------- 
uint8_t LCD1602A_u8Init(void) {

  // switch on the display
  LCD1602A_PWR = 0;
  
  // preset the control lines
  LCD1602A_ENA = 0;
  LCD1602A_RS = 0;
  LCD1602A_RW = 0; 

  // we wait 100ms (value is an assumption) for display start up
  TIMER0_vDelay(10u, e10ms); 
  
  // set the data bus to 8bit width, two lines
  // and font 5x8 
  // (first command without busy check -> we don't need the return value)
  (void) u8SendCmd(0x38, false); 

  // we wait 100us
  TIMER0_vDelay(1u, e100us);

  // clear the display
  if ( u8SendCmd(0x01, true) ) {
	return(1u);
  }

  // we wait 1600us
  TIMER0_vDelay(16u, e100us); 

  // cursor shifts right after input
  if ( u8SendCmd(0x06, true) ) {
	return(1u);
  }

  // we wait 100us
  TIMER0_vDelay(1u, e100us); 

  // switch the display on
  if ( u8SendCmd(0x0C, true) ) {
	return(1u);
  } 

  // we wait 100us
  TIMER0_vDelay(1u, e100us);

  return(0);
}


//----------------------------------------------------------------------
// Write a character u8Char in u8Row and  u8Colume
//
// returns 0 in case of success
// --------------------------------------------------------------------- 
uint8_t LCD1602A_u8WriteChar(uint8_t u8Row, uint8_t u8Column, uint8_t u8Char) {
  uint8_t u8Adr = 0x80; // bit 7 is fixed 1 for the address command 
   
  // first we check the inputs: we have two lines with 16 characters each
  if ( (u8Row >= mg_u8MaxRows) || (u8Column >= mg_u8MaxColumns) ) {
	return(1u);
  } 

  // compose the RAM address
  // First line address: 0 - 15
  // Second line address: 64 - 79 
  if ( u8Row ) { 
	// second line
	u8Adr |= (64u + u8Column);
  }
  else {
	// first line
	u8Adr |= u8Column; 
  }

  // write out the address
  if ( u8SendCmd(u8Adr, true) ) {
	return(1u);
  } 

  // we wait 100us
  TIMER0_vDelay(1u, e100us); 
  
  // write the character 
  if ( u8SendDisplayData(u8Char) ) {
	return(1u);
  }

  return(0);
}


//----------------------------------------------------------------------
// Clear u8Row (or better: fill it with spaces)
//
// returns 0 in case of success
// --------------------------------------------------------------------- 
uint8_t LCD1602A_u8ClearRow(uint8_t u8Row) {
  uint8_t u8i;
  uint8_t u8Adr = 0x80; // bit 7 is fixed 1 for the address command

  // first we check the input: we have only two lines
  if ( u8Row >= mg_u8MaxRows ) {
	return(1u);
  }

  // compose the RAM address
  // First line address: 0 - 15
  // Second line address: 64 - 79 
  if ( u8Row ) { 
	// second line
	u8Adr |= 64u;
  }

  // write out the address
  if ( u8SendCmd(u8Adr, true) ) {
	return(1u);
  } 

  // we wait 100us
  TIMER0_vDelay(1u, e100us); 
  
  // write the line
  for (u8i = 0; u8i < mg_u8MaxColumns; u8i++) {
	// write spaces (ASCII value 0x20)
	if ( u8SendDisplayData(0x20) ) {
	  return(1u);
	}
	// we wait 100us
	TIMER0_vDelay(1u, e100us); 
  } 

  return(0); 
} 


//----------------------------------------------------------------------
// Write a string in u8Row starting with u8Colume
//
// returns 0 in case of success
// --------------------------------------------------------------------- 
uint8_t LCD1602A_u8WriteString(uint8_t u8Row, uint8_t u8Column, uint8_t * pu8Char) {
  // bit 7 is fixed 1 for the address command 
  uint8_t u8Adr = 0x80; 

  // first we check the inputs: we have two lines with 16 characters each
  if ( (u8Row >= mg_u8MaxRows) || (u8Column >= mg_u8MaxColumns) ) {
	return(1u);
  }

  // compose the RAM address
  // First line address: 0 - 15
  // Second line address: 64 - 79 
  if ( u8Row ) { 
	// second line
	u8Adr |= (64u + u8Column);
  } else {
	// first line
	u8Adr |= u8Column; 
  }

  // write out the address
  if ( u8SendCmd(u8Adr, true) ) {
	return(1u);
  } 

  // we wait 100us
  TIMER0_vDelay(1u, e100us);

  // write the string
  // write until the null termination
  while (*pu8Char) { 
	// send the character
	if ( u8SendDisplayData(*pu8Char) ) {
	  return(1u);
	} 
	// calculate the next column
	u8Column++;
	// we do not write over the column limit
	if (u8Column >= mg_u8MaxColumns) {
	  break;
	}
	// prepare the next character
	pu8Char++;
  
	// we wait 100us
	TIMER0_vDelay(1u, e100us);
  }
  return(0); 
} 


//----------------------------------------------------------------------
// Switch off the display and set all ports and pins to a state
// where no light is on the display
// --------------------------------------------------------------------- 
void LCD1602A_vSwitchOff(void) {
  
  // switch off the display
  LCD1602A_PWR = 1u;
  
  // reset the control lines
  LCD1602A_ENA = 0;
  LCD1602A_RS = 0;
  LCD1602A_RW = 0; 

  // park the port at high level 
  LCD1602A_PORT = 0x00; 
}


//----------------------------------------------------------------------
// check if the parallel display bus is busy
// returns 1 if the busy flag in the display controller is set
// ---------------------------------------------------------------------
uint8_t u8CheckBusBusy(void) {
  uint16_t u16i;

  // check the LCD busy or not - with time out
  for (u16i = 0; u16i < 5000u; u16i++) {
    if ( (LCD1602A_PORT & 0x80) == 0 ) {
      return(0);
    }
  }
  return(1);
} 


//----------------------------------------------------------------------
// send a command with or without busy check
// 
// returns 0 in case of success 
// --------------------------------------------------------------------- 
uint8_t u8SendCmd(uint8_t u8Cmd, bool bWithBusyCheck) {
  // timing data from datasheet: 
  // Shenzhen EONE Electronics Co., LTD LCD1601A-1 (V1.2) 
  
  // the first command is send without bus busy check but
  // normally we make a busy check
  if (bWithBusyCheck) {
	// first we read the busy bit
	// set as command input
	LCD1602A_RS = 0;
	// set data bus into read mode
	LCD1602A_RW = 1;
	// park the port at high level 
	LCD1602A_PORT = 0xFF; 
	// prepare a high low slope on ENA 
	LCD1602A_ENA = 1; 
	// setup time for ENA high (enable pulse width must be > 140ns) 
	vDelay500ns(); 

	// check the LCD busy or not.  
	if ( u8CheckBusBusy() ) {
	  return(1u);
	} 
	// generate a slope -> data bus serves the data to read 
	LCD1602A_ENA = 0;
  }
  
  // set data bus into write mode 
  LCD1602A_RW = 0;
  // set the port value 
  LCD1602A_PORT = u8Cmd;
  // prepare a high low slope on ENA 
  LCD1602A_ENA = 1;
  // setup time for ENA high (enable pulse width must be > 140ns) 
  vDelay500ns(); 
  // generate a slope -> write out the command 
  LCD1602A_ENA = 0;
  // data hold time for the bus (must be > 10ns, we spend here 500ns to fullfill the 1.2us full cycle time ) 
  vDelay500ns(); 
  // park the port at high level 
  LCD1602A_PORT = 0xFF; 

  return(0);
}


//----------------------------------------------------------------------
// send one byte of display data
//
// returns 0 in case of success
// --------------------------------------------------------------------- 
uint8_t u8SendDisplayData(uint8_t u8Data) {

  // first we read the busy bit
  // set as command input
  LCD1602A_RS = 0;
  // set data bus into read mode
  LCD1602A_RW = 1;
  // park the port at high level 
  LCD1602A_PORT = 0xFF; 
  // prepare a high low slope on ENA 
  LCD1602A_ENA = 1; 
  // setup time for ENA high (enable pulse width must be > 140ns) 
  vDelay500ns(); 

  // check the LCD busy or not.  
  if ( u8CheckBusBusy() ) {
	return(1u);
  } 
  // generate a slope -> data bus serves the data to read 
  LCD1602A_ENA = 0; 

  // now we write the byte
  // set data bus into write mode 
  LCD1602A_RW = 0; 
  // set as data input
  LCD1602A_RS = 1; 
  // set the port value 
  LCD1602A_PORT = u8Data; 
  // prepare a high low slope on ENA 
  LCD1602A_ENA = 1;
  // setup time for ENA high (enable pulse width must be > 140ns) 
  vDelay500ns(); 
  // generate a slope -> write out the command 
  LCD1602A_ENA = 0;
  // data hold time for the bus (must be > 10ns, we spend here 500ns to fullfill the 1.2us full cycle time ) 
  vDelay500ns(); 
  // park the port at high level 
  LCD1602A_PORT = 0xFF; 

  return(0);
}


//----------------------------------------------------------------------
// delay used for timing the parallel display bus
// ---------------------------------------------------------------------
void vDelay500ns(void) {

  const uint8_t u8NopsFor500ns = (uint8_t) (UC_u32OscFreq / 2000000u); 
  uint8_t u8i;

  for (u8i = 0; u8i < u8NopsFor500ns; u8i++) {
	__asm nop __endasm;
	//__asm__ ("nop");
  }
}

