/***********************************************************************
 * Poti.c
 * Sound level potentiometer with seven relays
 * 
 ***********************************************************************
 * License
 * (C) Copyright 2018 Falk Richter, falk.richter at yandex.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#include <stdint.h>
#include "HAL_Settings.h"
#include "Poti.h"

static uint8_t mg_u8SoundLevel;
static uint8_t mg_u8WritePortsRequest;

//----------------------------------------------------------------------
// initialize the potentiometer module with default values
//---------------------------------------------------------------------- 
void POTI_vInit(uint8_t u8DefaultSoundLevel) {
  // if we get a strange default value -> we set it to mute
  if (u8DefaultSoundLevel > 127u) {
	u8DefaultSoundLevel = 0;
  }
  
  // set the default or the saved sound level
  mg_u8SoundLevel = u8DefaultSoundLevel;

  // write it out to the port
  POTI_PORT = mg_u8SoundLevel;
}


//----------------------------------------------------------------------
// set the physical ports at multiple changes only once and with a
// delay  
//---------------------------------------------------------------------- 
void POTI_vDoWork (void) {

  // delay counter variable
  static uint16_t u16DelayCounter;

  // check for a new write request
  if ( mg_u8WritePortsRequest ) {
	// reset the request
	mg_u8WritePortsRequest = 0;
	// start the delay counter
	u16DelayCounter = 0;
	// no further work in this step we return immediately
	return;
  } 
  
  // check for the finish delay time 
  if ( u16DelayCounter < 10000u ) {
	// delay time isn't over we increment the counter
	u16DelayCounter++;
  } else if ( u16DelayCounter == 10000u ) {
	// we increment the counter once again to
	// set the physical ports only one time
	u16DelayCounter++; 
	// write it out to the port
	POTI_PORT = mg_u8SoundLevel;
  }
}


//----------------------------------------------------------------------
// modify the sound level (stepwise or mute)
//---------------------------------------------------------------------- 
void POTI_vChangeSoundLevel( POTI_enAction_t enAction, uint8_t u8CmdType) { 

  static uint8_t u8LastCmdType;

  // my remote control sends after the command immediately a repeated command
  // so we ignore the first repeated command after a command
  if ( (u8LastCmdType == 1) && (u8CmdType == 2) ) {
	u8LastCmdType = u8CmdType;
	return;
  }

  // save the current command for the next run
  u8LastCmdType = u8CmdType; 

  switch(enAction) {
	case eIncrSoundLevel:
	  {
		if ( mg_u8SoundLevel < 127u ) {
		  // raise the sound level
		  mg_u8SoundLevel++;
		  // place a write request to the physical ports
		  mg_u8WritePortsRequest = 1u; 
		}
		break;
	  }
	case eDecrSoundLevel:
	  {
		if ( mg_u8SoundLevel > 0 ) {
		  // reduce the sound level
		  mg_u8SoundLevel--;
		  // place a write request to the physical ports
		  mg_u8WritePortsRequest = 1u;
		}
		break;
	  }
	case eMuteSoundLevel:
	  {
		// set the sound level to zero
		mg_u8SoundLevel = 0;
		// place a write request to the physical ports
		mg_u8WritePortsRequest = 1u;
		break;
	  }
  }
}


//----------------------------------------------------------------------
// get the actual sound level
//---------------------------------------------------------------------- 
uint8_t POTI_u8GetSoundLevel(void) {
  return(mg_u8SoundLevel);
}


//----------------------------------------------------------------------
// set the sound level directly
//---------------------------------------------------------------------- 
/*
void POTI_vSetSoundLevel(uint8_t u8SoundLevel) {
  
  // 7 bit is our highest level so if we get a strange
  // sound level we set it better to mute
  if (u8SoundLevel > 127u) {
	u8SoundLevel = 0;
  }
  
  // copy the new value into the module global variable
  mg_u8SoundLevel = u8SoundLevel;
  
  // set sound level
  POTI_PORT = mg_u8SoundLevel;
}
*/


//----------------------------------------------------------------------
// set the relays into standby
//---------------------------------------------------------------------- 
void POTI_vSetStandBy(void) {

  // we don't change the sound level here,
  // we set only all outputs to high to save the energy for the relays
  POTI_PORT = 127u;
}

