/**********************************************************************
 * Timer0.c
 * Timer 0 is used for 100us clock and as delay timer
 * 
 ***********************************************************************
 * License
 * (C) Copyright 2018 Falk Richter, falk.richter at yandex.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#include <stdint.h>
#include <stdbool.h>

#include "HAL_Settings.h"
#include "Timer0.h"

// global 100us timer tick flag
bool TIMER0_b100usTimeTick; 

// delay timer counter variable
static uint8_t mg_u8Counter100us;
static uint8_t mg_u8Counter10ms;
static uint8_t mg_u8Counter1s; 

// sub function of the delay timer
static uint8_t u8GetDiff (uint8_t u8StartValue, TIMER0_enType_t enType);

//----------------------------------------------------------------------
// initialize timer 0 to generate 100us ticks
//----------------------------------------------------------------------
void TIMER0_vInit(void) { 
  // systick timer runs with 1/10000=100us
  const uint16_t u16SysTick = 10000u;
  
  // first calculate the timer reload value
  // value = osc / systick + 0.5 (0.5 to round to the next full integer after casting)
  // here is the formula written to never leave the integer range
  const uint32_t u32TimerClocks = ( UC_u32OscFreq + u16SysTick / 2u ) / u16SysTick; 
  const uint16_t u16TimerReload =	(uint16_t) (((uint32_t) 65536u) - u32TimerClocks); 

  AUXR = 0x80;	//Timer0 set as 1T, 16 bits timer auto-reload, 
  TH0 = (uint8_t) (u16TimerReload / 256u);
  TL0 = (uint8_t) (u16TimerReload % 256u);
  ET0 = 1;	// Timer0 interrupt enable
  TR0 = 1;	// run Timer0

  // reset the timer tick flag
  TIMER0_b100usTimeTick = false;

  // preset the counter variables
  mg_u8Counter100us = 0;
  mg_u8Counter10ms = 0;
  mg_u8Counter1s = 0;
}


//----------------------------------------------------------------- 
// A blocking delay timer
//
// u8Time:
// Delay time (from 1 to 99) as factor of timer type  
// (e.g. e100us type: 1 = 100us delay) 
// 
// enType:
// timer type (resolution 100us, 10ms or 1s)
// ----------------------------------------------------------------
void TIMER0_vDelay (uint8_t u8Time, TIMER0_enType_t enType) {
  uint8_t u8StartVal;

  // the time value must be between 1 and 99
  if ( (u8Time == 0) || (u8Time > 99u) ) {
	// no time delay
	return;
  }

  // get the current value of the tick timer as start value
  switch (enType) {
	case e100us:
	  {
		u8StartVal = mg_u8Counter100us;
		break;
	  }
	case e10ms:
	  {
		u8StartVal = mg_u8Counter10ms;
		break; 
	  }
	case e1s:
	  {
		u8StartVal = mg_u8Counter1s; 
		break;
	  }
	default:
	  {
	  u8StartVal = mg_u8Counter100us;
		break;
	  }
  }

  while ( u8GetDiff( u8StartVal, enType ) < u8Time);
}


//----------------------------------------------------------------- 
// Function delivers the difference between the current value of 
// the time tick counter and the overtaken start value
//
// u8StartVal: 
// start value
// 
// enType:
// timer type (resolution 100us, 10ms or 1s)
//
// returns: time tick difference value
// ---------------------------------------------------------------- 
uint8_t u8GetDiff (uint8_t u8StartVal, TIMER0_enType_t enType) {
  uint8_t u8CurrentVal;
  uint8_t u8ReturnVal;

  // get the current value of the tick timer
  // we make a copy for an atomic access
  switch (enType) {
	case e100us:
	  {
		u8CurrentVal = mg_u8Counter100us;
		break;
	  }
	case e10ms:
	  {
		u8CurrentVal = mg_u8Counter10ms;
		break; 
	  }
	case e1s:
	  {
		u8CurrentVal = mg_u8Counter1s; 
		break;
	  }
	default:
	  {
		u8CurrentVal = mg_u8Counter100us;
		break;
	  }
  } 

  if ( u8CurrentVal >= u8StartVal ) { 
    // no overflow of the tick timer -> we can return the simple difference
    u8ReturnVal = u8CurrentVal - u8StartVal;
  } else {
    // the current value is smaller than the start value
    // -> the timer must be overflown
    // give back 99 - StartValue + CurrentValue
    u8ReturnVal = 99u - u8StartVal + u8CurrentVal;
  }
 
  return( u8ReturnVal );
}


//----------------------------------------------------------------------
// Timer0 interrupt
// ---------------------------------------------------------------------
void TIMER0_vInterrupt(void) __interrupt (1) {
  // 100us are over	
  TIMER0_b100usTimeTick = true; 

  // the counters for the delay timer
  // the 100us counter
  if ( mg_u8Counter100us < 99u ) {
	mg_u8Counter100us++;
  } else {
	// 100*100us = 10ms are over
	mg_u8Counter100us = 0;
	// the 10ms counter
	if ( mg_u8Counter10ms < 99u ) {
	  mg_u8Counter10ms++;
	} else {
	  // 100*10ms = 1s are over
	  mg_u8Counter10ms = 0;
	  // the 1s counter
	  if ( mg_u8Counter1s < 99u ) { 
		mg_u8Counter1s++;
	  } else {
		// 100*1s = 100s are over
		mg_u8Counter1s = 0;
	  }
	}
  }
}

